/*---------------------------------------------------------------------------------------------------*\
|   PROGRAM NAME: rx_attach_keys.sas                                                                  |
|     Attaches DrugClass and GenericName keys to the Dispensing table rows                            |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Version: Revised 1.0                                                                              |
|   Date: October 5, 2015                                                                             |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|       Dispensing records processed by RX_ETL                                                        |
|       Flattened NDC lookup table with multiple Drug Class and Generic Name keys                     |
|       Macro variables MaxDrugClass and MaxGenericName from program rx_lookup_keys.sas               |
|   Program outputs:                                                                                  |
|       Multiple tables written to DPTemp, named RxDrugClassYYY1, RxDrugClassYYY2,                    |
|         RxGenericNameYYY1, RxGenericNameYYY2, etc.                                                  |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
\*---------------------------------------------------------------------------------------------------*/

%** MODIFICATIONS
*** 10/8/15 - MR (1) Use LEAVE statement to breakout of loops as soon as it is safe to do so;
***              (2) Copy SORTEDBY value from input to output datasets;
***              (3) Retrieve key macro variables from metadata file;
***              (4) Rename macro to be same as file name;
***              (5) Add AgeEventGroupKey to core vars list;
*** 10/11/15- RR (1) Moved statement on list of variables to be saved in each dataset out of loop;
*** 10/20/15- RR (1) Added 'AgeQuarterKey' as variable that is saved in output datasets;
*** 10/22/15- RR (1) Ensure that PatKey is a 'core' saved variable in output datasets;
***              (2) Remove NDC as a 'core' saved variable in output datasets;
*** 12/08/15- MR (1) Read from single SAS view DPTemp.RXV instead of from time-split SAS datasets;
***              (2) Modify sortby logic to read from DPTemp.RXV;
*** 1/8/16- RR   (1) Ensure that SORTEDBY variables are only those kept in the datasets;


%**************** Link the yearly extracts with the NDC lookup table;
%macro Rx_Attach_Keys;
  %*** Declare local macro variables;
  %local SortedByRx MaxDrugClass MaxGenericName DrugClassKeyVars GenericNameKeyVars CoreRxVars RxDsNames year ;

  %*** Retrieve Macro Variables on max number of DrugClass and GenericName keys per NDC;
  data _null_;
    set DPtemp.MaxDrugsMeta;
    call symputx(name, value, 'L');
  run ;

  %*** Create list of Drug Class key and Generic Name key variables;
  %let DrugClassKeyVars=;
  %do m=1 %to &MaxDrugClass.;
    %*** Declare local variables;
    %local DrugClassKey&M.;
    %let DrugClassKeyVars=&DrugClassKeyVars. DrugClassKey&M.;
  %end;

  %let GenericNameKeyVars=;
  %do m=1 %to &MaxGenericName.;
    %local GenericKey&M.;
    %let GenericNameKeyVars=&GenericNameKeyVars. GenericNameKey&m.;
  %end;

  %*** Create list of variables to be saved in each dataset;
  %let CoreRxVars=RxID RxDate RxSup AgeTimePeriodGroupKey AgeEventGroupKey Sex Period Quarter AgeQuarterKey PatKey;
  %put &=CoreRxVars.;

  %* MOD 10/6/15: Determine SORTEDBY attribute for input dataset and store in macro variable;
  %* MOD 1/8/16:  Get only the variable names without assembling a full SORTEDBY expression;
  %let SortedByRx=%retrieve_sortedby(DSName=DPTemp.RXV,FullExpr=N);
  %put &=SortedByRx.;
  %* MOD 1/8/16: Check that SORTEDBY variables are in CoreRxVars and remove if not.  Then set the full SortedBy expression;
  %SortedBy_check(
    SourceWrds=&SortedbyRx.,
    ReturnVar=SortedByRx,
    CheckWrds=&CoreRxVars.
  );
  %** If there are SortedBy variables, then create full data set option expression;
  %if %length(&SortedByRx.)>0 %then %let SortedByRx=%str(SortedBy=&SortedByRx.);
  %put &=SortedByRx.;

  %*** Assemble output dataset names that get created after linking with Rx lookup table;
  %let RxDsNames=;
  %do year=&DP_StartYear. %to &DP_EndYear.;
    %*** Create output dataset names;
    %let RxDsNames= &RxDsNames. DPTemp.RxDrugClass&Year.(keep=&CoreRxVars. DrugClassKey &SortedByRx.)
                                DPTemp.RxGenericName&Year.(keep=&CoreRxVars. GenericNameKey &SortedByRx.);
  %end;
  %** MOD 10/11/15: Moved statement to below loop;
  %let RxDsNames= %cmpres(&RxDsNames);

  %*** Main data step for linking with Rx Lookup table;
  data &RxDsNames.;
    ** Hash join loading ;
    if _n_ eq 0 then do ;
      set DPTemp.NDC_Lookup_Flat(keep=NDC &DrugClassKeyVars. &GenericNameKeyVars.);
    end ;
    if _n_ eq 1 then do ;
      declare hash NDCLink(hashexp:16, dataset:"DPTemp.NDC_Lookup_Flat");
      NDCLink.definekey("NDC");
      NDCLink.definedata(%insertchar(%quotelst(&DrugClassKeyVars. &GenericNameKeyVars.),char=%str(,)));
      NDCLink.definedone();
    end ;

    %*** Establish arrays;
    array DC [*] DrugClassKey1-DrugClassKey&MaxDrugClass.;
    array GN [*] GenericNameKey1-GenericNameKey&MaxGenericName.;

    ** Read in source dataset ;
    do until(EOF);
      set DPTemp.RXV end=eof;

      %*** When there is an NDC link between Dispensing utilization and the lookup table...;
      if NDCLink.find()=0 then do;
        %*** Loop through DrugClass keys;
        do dck=1 to &MaxDrugClass.;
          %*** When the DrugClassKey value is valued, set variable and output record;
          if DC[dck] NE . then do;
            DrugClassKey=DC[dck];
            select (Period);
              %do year=&DP_StartYear. %to &DP_EndYear;
                when ("&year.") output DPTemp.RxDrugClass&year. ;
              %end;
              otherwise;
            end;
          end;
          else do ;
            %*** Break out of loop because this NDC does not have any additional DrugClasses...;
            leave ;
          end ;
        end;

        %*** Loop through GenericName keys;
        do gnk=1 to &MaxGenericName.;
          %*** When the GenericNameKey value is valued, set variable and output record;
          if GN[gnk] NE . then do;
            GenericNameKey=GN[gnk];
            select (Period);
              %do year=&DP_StartYear. %to &DP_EndYear;
                when ("&year.") output DPTemp.RxGenericName&year. ;
              %end;
              otherwise;
            end;
          end;
          else do ;
            %*** Break out of loop because this NDC does not have any additional GenericNames...;
            leave ;
          end ;
        end;
      end;
    end;
  run;
%mend Rx_Attach_Keys;
%Rx_Attach_Keys;


* End of rx_attach_keys.sas;
