****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: Create RX episode claims
*
* Created (mm/dd/yyyy): 11/26/2015
* Last modified:
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* This macro is an adaptation of the QPR 2.0.8 macro MS_CREATECLAIMEPI.SAS
*
* PURPOSE:
*   This program assigns episode numbers to dispensing claims based an allowable gap tolerance
*
*  Program inputs:
*    -Dataset containing the claims to create episodes with
*    -Input dataset must have the variables &PATID &GROUPING &CLMDATE &EXPIREDT &ENRSTART
*    -Input dispensing claims with invalid values (e.g. RxSup>0) must be excluded
*    -Input dispensing claims must be stockpiled
*  Program outputs:
*    -Dataset containing the created episode claims
*
* PARAMETERS:
*   -INFILE       = Dataset containing the claims to create episodes with
*   -GROUPING     = Variables in INFILE used to group claims
*   -GAP          = Maximum allowable gap between two claim (based on &CLMDATE &EXPIREDT &ENRSTART)
*   -PATID        = Name of variable in INFILE to identify the patient-ID (defaults to PatID)
*   -CLMDATE      = Name of variable in INFILE to identify the claim date (defaults to RxDate)
*   -EXPIREDT     = Name of variable in INFILE to identify the claim EndDt (defaults to ExpireDt)
*   -ENRSTART     = Name of variable in INFILE to identify enrollment span StartDt (defaults to EStart)
*   -OUTFILE      = Dataset containing the episodes
*   -DEBUG        = Y/N flag for producing extra output (default is N)
*
*  Programming Notes:
*   -This assigns an episode variable to dispensing claims, how one rolls them up is external to
*    this macro
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO:
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG:
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*             mm/dd/yy
*
***************************************************************************************************;

/* MOD: 10/14/15 -MR: */
/*                1) Modify to issue notification if overlapping claims are encountered */

%macro CREATE_RX_EPISODE_CLAIMS
  ( INFILE=,
    GROUPING=,
    GAP=,
    PATID=,
    CLMDATE=,
    EXPIREDT=,
    ENRSTART=,
    OUTFILE=,
    DEBUG=N
   );

%put =====> MACRO CALLED: CREATE_RX_EPISODE_CLAIMS.SAS;

    /*Set default values for blank parameters*/
    %if %length(&PATID)=0 %then %do;
       %let PATID = PatID;
    %end;
    %if %length(&CLMDATE)=0 %then %do;
       %let &CLMDATE = RxDate;
    %end;
    %if %length(&EXPIREDT)=0 %then %do;
       %let &EXPIREDT = ExpireDt;
    %end;
    %if %length(&ENRSTART)=0 %then %do;
       %let &ENRSTART = EStart;
    %end;

    %put *** Display Parameter Values ;
    %put _local_ ;
    %put ;

    *** Determine the group-by variables used for creating episodes;
    %local DIMS ;
    %let DIMS = &PATID &GROUPING.;
    %put &=DIMS.;
    %put ;

    *** Determine right-most variable in group-by variables;
    %local DIMCNT LASTDIM;
    %let DIMCNT = %SYSFUNC(COUNTW(&DIMS.));
    %let LASTDIM = %SCAN(&DIMS.,&DIMCNT.) ;
    %put &=LASTDIM.;
    %put ;

    /* WhichFile local macro variable is conditionally set to &OUTFILE or &INFILE */
    /*   WhichFile is set to &OUTFILE when */
    /*     - &INFILE is NOT sorted properly */
    /*   WhichFile is set to &INFILE when: */
    /*     - &INFILE IS sorted properly */
    %local WhichFile ;
    *** Verify &INFILE sort order;
    %local SORTEDBY N_SORTED;
    %let N_SORTED = %SYSFUNC(COUNTW(&PATID. &GROUPING. &CLMDATE.));
    %*put &=N_SORTED;
    %*put ;
    proc contents noprint data=&INFILE. out=_contents_ (keep=name sortedby) ;
    run ;
    proc sql noprint ;
      select upcase(name) into :SORTEDBY separated by ' '
        from _contents_
        where sortedby between 1 and &N_SORTED.
        order by sortedby
      ;
    quit ;
    %put ;
    %put &=SORTEDBY ;
    %put ;
    %if %upcase(%cmpres(&SORTEDBY)) EQ %upcase(%cmpres(&PATID. &GROUPING. &CLMDATE.))  %then %do;
        *** &INFILE is already sorted in the proper order;
        %let WhichFile=&INFILE. ;
        %put &=WhichFile &=INFILE ;
    %end;
    %else %do ;
        *** Sort &INFILE by &PATID. &GROUPING.  &CLMDATE. outputting to &OUTFILE.;
        proc sort data=&INFILE. out=&OUTFILE. ;
          by &PATID. &GROUPING. &CLMDATE. ;
        run ;
        %let WhichFile=&OUTFILE. ;
        %put &=WhichFile &=INFILE ;
    %end;

    *** Assign episode numbers to claims;
    data &outfile.;
    set &WhichFile. end=lastObs;
    by &PATID. &GROUPING. &CLMDATE. ;

   retain episode LRunOutDate LEnrStart .;
   length episode 3. LRunOutDate LEnrStart 4.;
   format episode 3. LRunOutDate LEnrStart mmddyy10.;

   if first.&LASTDIM. then do;
      LRunOutDate=&EXPIREDT.;
      LEnrStart=&ENRSTART.;
      episode=1;
      gap=.;
   end;
   else do;
      gap = (&CLMDATE - LRunOutDate)-1;
      if (gap > &Gap.) or (lEnrStart ne &ENRSTART.) then do;
          episode=episode+1;
          LRunOutDate = &EXPIREDT.;
      end;
      else do;
         if LRunOutDate > &EXPIREDT. then do;
            _trap_+1;
            if _trap_ eq 1 then do;
               put 'Overlapping claims detected which could be a sign of problems';
               put 'Please check upstream cleaning/stockpiling processes ...';
            end;
         end;
         LRunOutDate = max(&EXPIREDT.,LRunOutDate);
      end;
   end;
   LEnrStart = &ENRSTART.;
   %if %upcase(&DEBUG.) ne Y %then %do;
      drop LRunOutDate LEnrStart gap _trap_;
   %end;
   if lastObs then do;
      put;
      put 'Total # of overlapping claims detected: ' _trap_ ;
      put;
   end ;
run;

%put NOTE: ********END OF MACRO: CREATE_RX_EPISODE_CLAIMS.SAS ********;

%mend CREATE_RX_EPISODE_CLAIMS;

* End of create_rx_episode_claims.sas;