/*---------------------------------------------------------------------------------------------------*\
|   PROGRAM NAME: utility_macros.sas                                                                  |
|   A place for defining general purpose utility macros used for performing administrative tasks      |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Version: Revised 1.0                                                                              |
|   Date: August 27, 2015                                                                             |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|       NONE                                                                                          |
|                                                                                                     |
|   Program outputs:                                                                                  |
|       compiled macros                                                                               |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
\*---------------------------------------------------------------------------------------------------*/

%**************** BEGIN LIST OF MACROS INCLUDED ***********************************************;
%*;
%*  Dsn_Exists          - Checks if SAS dataset exists;
%*  Dsn_Obs             - Returns number of obs for a SAS dataset;
%*  Get_Names           - Retrive list of file or directory names and store in a SAS dataset;
%*  InsertChar          - Inserts delimiter between words in a string;
%*  NumLength           - Returns smallest # of bytes needed to accurately store an interger;
%*  QuoteLst            - Inserts quotes around words in a list;
%*  Retrieve_SortedBy   - Returns the SORTEDBY value of a SAS dataset;
%*  TimeReport          - Prints to the log the elapsed time between two time values;
%*  TimeStamp           - Prints current time to log and optionally save to a macro variable;
%*;
%**************** END LIST OF MACROS INCLUDED *************************************************;

%MACRO DSN_EXISTS(dsn);
%*** Macro tests if SAS dataset exists;
   %local dsid exists rc;
   %let dsid = %sysfunc(open(&dsn));
   %let rc = %sysfunc(close(&dsid));
   %let exists=0;
   %if &dsid %then %do;
      %let exists = 1;
   %end;
   &exists
%MEND DSN_EXISTS;

%MACRO DSN_OBS(dsn);
%*** Macro retrieves number of observations in a SAS dataset;
   %local dsid obs rc;
   %let dsid = %sysfunc(open(&dsn));
   %if &dsid %then %do;
      %let obs = %sysfunc(attrn(&dsid,NOBS));
      %let rc = %sysfunc(close(&dsid));
      &obs
   %end;
   %else
      %put Open for data set &dsn failed - %sysfunc(sysmsg());
%MEND DSN_OBS;


%MACRO get_names(
    type = ,             /* Specify a type to search [dir_name | file_name]  */
    root=./     ,        /* Root to start search                             */
    out=_data_  ,        /* Output data set with search results              */
    source=     ,        /* Value for optional SOURCE variable               */
    filter=1             /* directory/file filter                            */
    );

/* Acknowledgement - This is an adaptation of a Jack Hamilton macro get_file_names   */
/*                   which searches for files. This adapted macro searches for either */
/*                   files or directories                                             */

    /* The MOPEN function returns a 0 for directories and a positive number for files. */
    %local found_type ;
    %if %lowcase(&type) eq dir_name %then %do ;
      %let found_type = eq ;    /* equal to 0 means a directory is found */
    %end ;
    %else %if %lowcase(&type) eq file_name %then %do ;
      %let found_type = gt ;    /* > 0 means a file is found */
    %end ;

    data &OUT.;

        keep name type root;
        length root $600.;
        retain type "&type" root "&root";

        %if %length(&SOURCE.) ne 0
        %then
            %do;
            retain source &SOURCE.;
            keep source;
            %end;

        length fref $8  name $600;
        rc = filename(fref, root);
        if rc = 0 then
          do;
            did = dopen(fref);
            rc = filename(fref);
        end;
        else
          do;
            length msg $200.;
            msg = sysmsg();
            put msg=;
            did = .;
          end;

        if did <= 0
        then
            putlog 'ERR' 'OR: Unable to open directory.';

        dnum = dnum(did);

        do i = 1 to dnum;
            name = dread(did, i);
            /* If this entry is the right type, then output. */
            fid = mopen(did, name);
            if fid &found_type 0
            then if &FILTER.
                then
                    output;
        end;

        rc = dclose(did);
        rc = filename(fref);

    run;
%MEND get_names ;

%MACRO InsertChar(wordlist,char=%str(,));
/**************************************************************************************************************
*                                                                                                             *
* NAME:  insertchar                                                                                           *
*                                                                                                             *
* TYPE:  SAS MACRO                                                                                            *
*                                                                                                             *
* DESC:  Inserts specified character within a string of words (not after the last or before the first word)   *
*                                                                                                             *
* USAGE: any where in a SAS Program                                                                           *
*                                                                                                             *
* EXAMPLE:                                                                                                    *
*   %let ClassVars=Age Sex State;                                                                             *
*   %InsertChar(&ClassVars.,char=%str(*)) would produce (no quotes) "Age*Sex*State"                           *
* PARMS:                                                                                                      *
* --------        --------------------------------------------------------------------------------------------*
*  wordlist       multi-word string                                                                           *
*  char           macro keyword parm of the character to insert in between words (default is a comma)         *
*=============================================================================================================*
* NEW:  11/02/04 - New (RRosofsky)                                                                            *
***************************************************************************************************************/
  %local numwrd string;
  %let numwrd=%sysfunc(countw(&wordlist));
  %* If only 1 word, then return string;
  %if &numwrd=1 %then &wordlist;
  %else %do;
    %let string=%str();
    %do i=1 %to %eval(&numwrd-1);
      %let string=&string %scan(&wordlist,&i) %str(&char);
    %end;
    %let string= &string %scan(&wordlist,&numwrd);
    &string
  %end;
%MEND;

%MACRO NumLength(NumIn);
%** Macro function to determine the minimum number of bytes to hold a numeric integer;
    %local _val ;

    %** Set default length to max length for safety;
    %let _val=8;

    %** Check if smaller length can be safely used;
    %if %eval(0 <= &NumIn) and %eval(&NumIn <= 8192) %then %do;
        %let _val=3;
        %end;
     %else %if %eval(8193 <= &NumIn) and %eval(&NumIn <= 2097152) %then %do;
        %let _val=4;
        %end;
     %else %if %eval(2097153 <= &NumIn) and %eval(&NumIn <= 536870912) %then %do;
        %let _val=5;
        %end;
     %else %if %eval(536870913 <= &NumIn) and %eval(&NumIn <= 137438953472) %then %do;
        %let _val=6;
        %end;
     %else %if %eval(137438953473 <= &NumIn) and %eval(&NumIn <= 35184372088832) %then %do;
        %let _val=7;
        %end;

    %** return smallest safe value to calling environment;
    &_val
%MEND;


%MACRO QuoteLst(str,quote=%str(%"),delim=%str( ));
  /*<pre><b>
  / Program   : quotelst.sas
  / Version   : 1.0
  / Author    : Roland Rashleigh-Berry
  / Date      : October 2002
  / Purpose   : To quote the elements of a list
  / SubMacros : none
  / Notes     : DO NOT COPY AND PASTE THIS FROM A BROWSER WINDOW. YOU MUST USE
  /             THE "VIEW" PULL-DOWN WINDOW AND USE "SOURCE". This is because the
  /             browser will change some of the characters in this file to quotes.
  /
  /             This is useful to turn a list into a quoted list so that you can
  /             use the in() function on it in a data step. Also, if you search for
  /             a quoted string among a list of quoted strings then you can avoid
  /             matching on a subset of a single element. Note that you can change
  /             not only the quote mark but the delimiter as well so you can use
  /             this macro for other purposes like putting commas between variable
  /             names etc. It is assumed that the elements of the list are
  /             delimited by spaces.
  / Usage     : %if %index(%quotelst(varnames),"varname") %then...
  /
  /===============================================================================
  / PARAMETERS:
  /-------name------- -------------------------description------------------------
  / str               String to quote elements of (pos)
  / quote=%str(%")    Quote character to use (defaults to double quotation mark)
  / delim=%str( )     Delimiter character to use (defaults to a space)
  /===============================================================================
  / AMENDMENT HISTORY:
  / init --date-- mod-id ----------------------description------------------------
  /
  /===============================================================================
  / This is public domain software. No guarantee as to suitability or accuracy is
  / given or implied. User uses this code entirely at their own risk.
  /===============================================================================
  / Accessed from http://www.datasavantconsulting.com/roland/quotelst.sas on 1/12/2007
  / by Robert Rosofsky                                                              */

  %local i quotelst;
  %let i=1;
  %do %while(%length(%scan(&str,&i,%str( ))) GT 0);
    %if %length(&quotelst) EQ 0 %then %let quotelst=&quote.%scan(&str,&i,%str( ))&quote;
    %else %let quotelst=&quotelst.&quote.%scan(&str,&i,%str( ))&quote.;
    %let i=%eval(&i + 1);
    %if %length(%scan(&str,&i,%str( ))) GT 0 %then %let quotelst=&quotelst.&delim;
  %end;
  %unquote(&quotelst)
%MEND;


%*****************************************************************************************************************************
* macro retrieve_sortedby
* Retrieves the SORTEDBY attribute of a dataset
* Returns it as part of a full expression (e.g., "SORTEDBY=Var1") or simply as the name of the variable(s) (e.g. "Var1")
* NOTE: If FullExpr=Y and there is no SORTEDBY attribute, then a null value will be returned as a data set option cannot
*       be "(SORTEDBY=)" with a blank list of variables
* NEW 10/11/2015 RRosofsky
* MOD 10/20/2015 RRosofsky: Modified how and what is returned from this macro, see NOTE above
******************************************************************************************************************************;
%MACRO Retrieve_SortedBy(
  DSName=,
  FullExpr=Y
  );
  %local dsid rc _SortedBy;
  %let dsid=%sysfunc(open(&DSName.,I));
  %let _SortedBy=%sysfunc(attrc(&dsid.,SORTEDBY));
  %let rc=%sysfunc(close(&dsid.));
  %put &DSName SORTEDBY option = &_SortedBy;
  %*** Return value alone or else full expression with "SortedBy=" phrase;
  %if %upcase(&FullExpr.)=Y and %length(&_SortedBy.) > 0 %then %do;
    %str(SORTEDBY=&_SortedBy.)
  %end;
  %else %do;
    &_SortedBy.
  %end;
%MEND;

%MACRO TimeReport(time1,time2);
  %********************************************************************************************************
  * Macro calculates the difference in time between 2 time values passed as parameters.  It does not matter
  *   which is later as this macro handles absolute values
  * Created: RRosofsky 9/21/2005
  *   time1 = Time value # 1
  *   time2 = Time value # 2
  * Modifications: 11/30/2005 RRosofsky: Put entirely in macro code, dropping a "data _null_" step
  *********************************************************************************************************;
  %** Set local mvars;
  %local errm1 ;

  %*** Error check that parms have been passed;
  %if &time1= or &time2= %then %let errm1=Missing parms passed ;
  %******** If any error message exists, print it to log and stop macro tool.  Otherwise continue;
  %if %length(&errm1.)> 0 %then %do;
    %put; %put;
    %put ***************************************************************************************************;
    %put *  !!!!! macro TimeReport is aborting - Note reasons below ;
    %do i=1 %to 1;
      %if %length(&&&errm&i)>0 %then %put *** &&&&errm&i;
    %end;
    %put ***************************************************************************************************;
    %put; %put;
  %end;
  %else %do;
    %**** Print to log the result;
    %put;
    %**** Take the absolute value of the difference in 2 time values and print in "time." format ;
    %put Time duration is %sysfunc(putn(%sysfunc(abs(%sysevalf(&&&time2. - &&&time1.))),time.));
    %put; %put;
  %end;
%MEND;


%MACRO TimeStamp(parm=);
  %********************************************************************************************************
  * Macro prints to log the current time.  Can be used for calculating time to complete a job
  * Created: RRosofsky 9/21/2005
  *   PARM = Name of macro parm (OPTIONAL)
  * Modifications:
  * 9/9/2015: Saves the macro variable as a full SAS date/time value
  * 9/15/2015: Creates global macro only if (a) parm is not missing and (b) the macro variable does not
  *            yet exist
  *********************************************************************************************************;
  %put;
  %put ***** Time now is %sysfunc(putn(%sysfunc(time()),timeampm8.));
  %** If a value to parm is passed, then save the time as a macro variable as well;
  %if %length(&parm) > 0 %then %do;
    %** if the macro variable does not already exist, create it as a global macro variable **;
    %if %symexist(&parm) eq 0 %then %do;
      %** Set as global parm;
      %global &parm;
    %end ;
    %** Set value to current time;
    %let &parm=%sysfunc(DateTime());
    %put **** Time value saved to macro variable %upcase(&parm) ;
  %end;
  %put ;
%MEND;

* End of utility_macros.sas;